@@ -3,7 +3,7 @@ functions.py - wraps functions of module _ldap
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: functions.py,v 1.23 2009/07/26 11:09:58 stroeder Exp $
+\$Id: functions.py,v 1.26 2010/06/03 12:26:39 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -26,9 +26,9 @@ __all__ = [
   'get_option','set_option',
 ]
 
-import sys,_ldap
+import sys,_ldap,ldap
 
-from ldap import _ldap_module_lock,LDAPError
+from ldap import LDAPError
 
 from ldap.dn import explode_dn,explode_rdn
 
@@ -37,34 +37,40 @@ from ldap.ldapobject import LDAPObject
 if __debug__:
   # Tracing is only supported in debugging mode
   import traceback
-  from ldap import _trace_level,_trace_file,_trace_stack_limit
 
-def _ldap_function_call(func,*args,**kwargs):
+
+def _ldap_function_call(lock,func,*args,**kwargs):
   """
-  Wrapper function which locks calls to func with via
-  module-wide ldap_lock
+  Wrapper function which locks and logs calls to function
+
+  lock
+      Instance of threading.Lock or compatible
+  func
+      Function to call with arguments passed in via *args and **kwargs
   """
+  if lock:
+    lock.acquire()
   if __debug__:
-    if _trace_level>=1:
-      _trace_file.write('*** %s.%s (%s,%s)\n' % (
+    if ldap._trace_level>=1:
+      ldap._trace_file.write('*** %s.%s (%s,%s)\n' % (
         '_ldap',repr(func),
         repr(args),repr(kwargs)
       ))
-      if _trace_level>=3:
-        traceback.print_stack(limit=_trace_stack_limit,file=_trace_file)
-  _ldap_module_lock.acquire()
+      if ldap._trace_level>=3:
+        traceback.print_stack(limit=ldap._trace_stack_limit,file=ldap._trace_file)
   try:
     try:
       result = func(*args,**kwargs)
     finally:
-      _ldap_module_lock.release()
+      if lock:
+        ldap._ldap_module_lock.release()
   except LDAPError,e:
-    if __debug__ and _trace_level>=2:
-      _trace_file.write('=> LDAPError: %s\n' % (str(e)))
+    if __debug__ and ldap._trace_level>=2:
+      ldap._trace_file.write('=> LDAPError: %s\n' % (str(e)))
     raise
-  if __debug__ and _trace_level>=2:
+  if __debug__ and ldap._trace_level>=2:
     if result!=None and result!=(None,None):
-      _trace_file.write('=> result: %s\n' % (repr(result)))
+      ldap._trace_file.write('=> result: %s\n' % (repr(result)))
   return result
 
 
@@ -72,7 +78,7 @@ def initialize(uri,trace_level=0,trace_file=sys.stdout,trace_stack_limit=None):
   """
   Return LDAPObject instance by opening LDAP connection to
   LDAP host specified by LDAP URL
-  
+
   Parameters:
   uri
         LDAP URL containing at least connection scheme and hostport,
@@ -90,7 +96,7 @@ def open(host,port=389,trace_level=0,trace_file=sys.stdout,trace_stack_limit=Non
   """
   Return LDAPObject instance by opening LDAP connection to
   specified LDAP host
-  
+
   Parameters:
   host
         LDAP host and port, e.g. localhost
@@ -102,6 +108,8 @@ def open(host,port=389,trace_level=0,trace_file=sys.stdout,trace_stack_limit=Non
         File object where to write the trace output to.
         Default is to use stdout.
   """
+  import warnings
+  warnings.warn('ldap.open() is deprecated! Use ldap.initialize() instead.', DeprecationWarning,2)
   return initialize('ldap://%s:%d' % (host,port),trace_level,trace_file,trace_stack_limit)
 
 init = open
@@ -113,13 +121,13 @@ def get_option(option):
 
   Get the value of an LDAP global option.
   """
-  return _ldap_function_call(_ldap.get_option,option)
+  return _ldap_function_call(None,_ldap.get_option,option)
 
 
 def set_option(option,invalue):
   """
   set_option(name, value)
-  
+
   Set the value of an LDAP global option.
   """
-  _ldap_function_call(_ldap.set_option,option,invalue)
+  return _ldap_function_call(None,_ldap.set_option,option,invalue)
