@@ -1,5 +1,5 @@
 /* See http://www.python-ldap.org/ for details.
- * $Id: LDAPObject.c,v 1.82 2009/04/17 12:19:09 stroeder Exp $ */
+ * $Id: LDAPObject.c,v 1.88 2009/10/08 18:22:44 stroeder Exp $ */
 
 #include "common.h"
 #include "patchlevel.h"
@@ -11,6 +11,7 @@
 #include "LDAPObject.h"
 #include "ldapcontrol.h"
 #include "message.h"
+#include "berval.h"
 #include "options.h"
 
 #ifdef HAVE_SASL
@@ -337,14 +338,12 @@ l_ldap_unbind_ext( LDAPObject* self, PyObject* args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -380,14 +379,12 @@ l_ldap_abandon_ext( LDAPObject* self, PyObject* args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -429,14 +426,12 @@ l_ldap_add_ext( LDAPObject* self, PyObject *args )
 	return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -474,14 +469,12 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -657,14 +650,12 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -719,14 +710,12 @@ l_ldap_cancel( LDAPObject* self, PyObject* args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -767,14 +756,12 @@ l_ldap_compare_ext( LDAPObject* self, PyObject *args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -810,14 +797,12 @@ l_ldap_delete_ext( LDAPObject* self, PyObject *args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -859,14 +844,12 @@ l_ldap_modify_ext( LDAPObject* self, PyObject *args )
 	return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -906,14 +889,12 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -975,41 +956,35 @@ l_ldap_result3( LDAPObject* self, PyObject *args )
     if (msg)
 	    res_msgid = ldap_msgid(msg);
 
-    if (res_type == LDAP_RES_SEARCH_ENTRY
-	    || res_type == LDAP_RES_SEARCH_REFERENCE
-	)
-	pmsg = LDAPmessage_to_python( self->ldap, msg );
-    else {
-	int result;
-	char **refs = NULL;
-        LDAPControl **serverctrls = 0;
-	LDAP_BEGIN_ALLOW_THREADS( self );
-	ldap_parse_result( self->ldap, msg, &result, NULL, NULL, &refs,
-                           &serverctrls, 0 );
-	LDAP_END_ALLOW_THREADS( self );
+    int result;
+    char **refs = NULL;
+    LDAPControl **serverctrls = 0;
+    LDAP_BEGIN_ALLOW_THREADS( self );
+    ldap_parse_result( self->ldap, msg, &result, NULL, NULL, &refs,
+		       &serverctrls, 0 );
+    LDAP_END_ALLOW_THREADS( self );
 
-	if (result != LDAP_SUCCESS) {		/* result error */
-	    char *e, err[1024];
-	    if (result == LDAP_REFERRAL && refs && refs[0]) {
-		snprintf(err, sizeof(err), "Referral:\n%s", refs[0]);
-		e = err;
-	    } else
-		e = "ldap_parse_result";
-	    ldap_msgfree(msg);
-	    return LDAPerror( self->ldap, e );
-	}
-
-        if (!(pyctrls = LDAPControls_to_List(serverctrls))) {
-            int err = LDAP_NO_MEMORY;
-            ldap_set_option(self->ldap, LDAP_OPT_ERROR_NUMBER, &err);
-	    ldap_msgfree(msg);
-            return LDAPerror(self->ldap, "LDAPControls_to_List");
-        }
-        ldap_controls_free(serverctrls);
-
-	pmsg = LDAPmessage_to_python( self->ldap, msg );
+    if (result != LDAP_SUCCESS) {		/* result error */
+	char *e, err[1024];
+	if (result == LDAP_REFERRAL && refs && refs[0]) {
+	    snprintf(err, sizeof(err), "Referral:\n%s", refs[0]);
+	    e = err;
+	} else
+	    e = "ldap_parse_result";
+	ldap_msgfree(msg);
+	return LDAPerror( self->ldap, e );
     }
 
+    if (!(pyctrls = LDAPControls_to_List(serverctrls))) {
+	int err = LDAP_NO_MEMORY;
+	ldap_set_option(self->ldap, LDAP_OPT_ERROR_NUMBER, &err);
+	ldap_msgfree(msg);
+	return LDAPerror(self->ldap, "LDAPControls_to_List");
+    }
+    ldap_controls_free(serverctrls);
+
+    pmsg = LDAPmessage_to_python( self->ldap, msg );
+
     result_str = LDAPconstant( res_type );
 
     if (pmsg == NULL) {
@@ -1077,14 +1052,12 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -1123,14 +1096,12 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
     if (!PyArg_ParseTuple( args, "|OO", &serverctrls, &clientctrls)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -1144,7 +1115,7 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
     if ( ldaperror!=LDAP_SUCCESS )
     	return LDAPerror( self->ldap, "ldap_whoami_s" );
 
-    result = PyString_FromStringAndSize(bvalue->bv_val, bvalue->bv_len);
+    result = LDAPberval_to_object(bvalue);
 
     return result;
 }
@@ -1231,14 +1202,12 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        server_ldcs = List_to_LDAPControls(serverctrls);
-        if (server_ldcs == NULL)
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
             return NULL;
     }
 
     if (!PyNone_Check(clientctrls)) {
-        client_ldcs = List_to_LDAPControls(clientctrls);
-        if (client_ldcs == NULL)
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
             return NULL;
     }
 
@@ -1284,9 +1253,6 @@ static PyMethodDef methods[] = {
     {"passwd",	        (PyCFunction)l_ldap_passwd,	        METH_VARARGS },
     {"set_option",	(PyCFunction)l_ldap_set_option,		METH_VARARGS },
     {"get_option",	(PyCFunction)l_ldap_get_option,		METH_VARARGS },
-#if defined(FILENO_SUPPORTED)
-    {"fileno",		(PyCFunction)l_ldap_fileno,		METH_VARARGS },
-#endif
 #ifdef LDAP_API_FEATURE_CANCEL
     {"cancel",		(PyCFunction)l_ldap_cancel,		METH_VARARGS },
 #endif
